import { useEffect } from 'react';
import { DialogProps } from '../../../components/layout/Dialog';
import React from 'react';
import { useAppDispatch, useQuerybuilderSettings } from '../../../data-access';
import { QueryBuilderSettings, setQuerybuilderSettings } from '../../../data-access/store/querybuilderSlice';
import { addWarning } from '../../../data-access/store/configSlice';
import { FormActions, FormBody, FormCard, FormControl, FormDiv, FormHBar, FormTitle } from '../../../components/forms';
import { Layouts } from '@graphpolaris/shared/lib/graph-layout';
import { Input } from '@graphpolaris/shared/lib/components/inputs';

type QuerySettingsDialogProps = DialogProps;

export const QuerySettingsDialog = React.forwardRef<HTMLDivElement, QuerySettingsDialogProps>((props, ref) => {
  const qb = useQuerybuilderSettings();
  const dispatch = useAppDispatch();
  const [state, setState] = React.useState<QueryBuilderSettings>(qb);

  useEffect(() => {
    setState(qb);
  }, [qb, props.open]);

  function submit() {
    if (state.depth.min < 0) {
      dispatch(addWarning('The minimum depth cannot be smaller than 0'));
    } else if (state.depth.max > 99) {
      dispatch(addWarning('The maximum depth cannot be larger than 99'));
    } else if (state.depth.min > state.depth.max) {
      dispatch(addWarning('The minimum depth cannot be larger than the maximum depth'));
    } else {
      dispatch(setQuerybuilderSettings(state));
      props.onClose();
    }
  }

  return (
    <>
      {props.open && (
        <FormDiv hAnchor="right" ref={ref}>
          <FormCard>
            <FormBody
              onSubmit={(e) => {
                e.preventDefault();
                submit();
              }}
            >
              <FormTitle title="Query Settings" onClose={props.onClose} />
              <FormHBar />
              <FormControl>
                <Input
                  type="boolean"
                  value={state.autocompleteRelation}
                  label="Autocomplete Edges"
                  tooltip="When enabled, if you drag a relationship to the query, the query builder will automatically add the entity nodes it is connected to."
                  onChange={(value: boolean) => {
                    setState({ ...state, autocompleteRelation: value as any });
                  }}
                />
              </FormControl>

              <FormHBar />
              <div className="form-control px-5">
                <label className="label">
                  <span className="label-text">Limit - Max number of results</span>
                </label>
                <input
                  type="number"
                  className="input input-sm input-bordered"
                  placeholder="500"
                  value={state.limit}
                  onChange={(e) => setState({ ...state, limit: parseInt(e.target.value) })}
                />
              </div>
              <FormHBar />
              <div className="form-control px-5 flex flex-row gap-3">
                <div className="">
                  <label className="label">
                    <span className="label-text">Min Depth Default</span>
                  </label>
                  <input
                    type="number"
                    className="input input-sm input-bordered w-full"
                    placeholder="0"
                    min={0}
                    max={state.depth.max}
                    value={state.depth.min}
                    onChange={(e) => setState({ ...state, depth: { min: parseInt(e.target.value), max: state.depth.max } })}
                    onKeyDown={(e) => {
                      if (e.key === 'Enter') {
                        submit();
                      }
                    }}
                  />
                </div>
                <div className="">
                  <label className="label">
                    <span className="label-text">Max Depth Default</span>
                  </label>
                  <input
                    type="number"
                    className="input input-sm input-bordered w-full"
                    placeholder="0"
                    min={state.depth.min}
                    max={99}
                    value={state.depth.max}
                    onChange={(e) => setState({ ...state, depth: { max: parseInt(e.target.value), min: state.depth.min } })}
                    onKeyDown={(e) => {
                      if (e.key === 'Enter') {
                        submit();
                      }
                    }}
                  />
                </div>
              </div>
              <FormHBar />
              <div className="form-control px-5 ">
                <label className="label">
                  <span className="label-text">Layout Type</span>
                </label>
                <select
                  className="select select-primary select-sm "
                  value={state.layout}
                  onChange={(e) => {
                    setState({ ...state, layout: e.target.value as any });
                  }}
                >
                  <option className="option" value={'manual'}>
                    Manual
                  </option>
                  {Object.entries(Layouts).map(([k, v]) => (
                    <option className="option" value={v} key={v}>
                      {k}
                    </option>
                  ))}
                </select>
              </div>
              <FormHBar />

              <FormActions onClose={props.onClose} />
            </FormBody>
          </FormCard>
        </FormDiv>
      )}
    </>
  );
});
